@@ -1,19 +1,17 @@
 package org.jmx4perl.backend;
 
-import org.jmx4perl.JmxRequest;
-import org.jmx4perl.handler.JsonRequestHandler;
-
-import javax.management.*;
-import javax.naming.InitialContext;
-import javax.naming.NamingException;
 import java.io.IOException;
 import java.lang.management.ManagementFactory;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
-import java.util.Collections;
-import java.util.LinkedHashSet;
-import java.util.List;
-import java.util.Set;
+import java.util.*;
+
+import javax.management.*;
+import javax.naming.InitialContext;
+import javax.naming.NamingException;
+
+import org.jmx4perl.JmxRequest;
+import org.jmx4perl.handler.JsonRequestHandler;
 
 /*
  * jmx4perl - WAR Agent for exporting JMX via JSON
@@ -45,7 +43,7 @@ import java.util.Set;
  * @author roland
  * @since Jun 15, 2009
  */
-public class MBeanServerHandler {
+public class MBeanServerHandler implements MBeanServerHandlerMBean,MBeanRegistration {
 
     // The MBeanServers to use
     // TODO: Dont cache them here ! Important for OSGi usage ...
@@ -54,10 +52,28 @@ public class MBeanServerHandler {
 
     // Whether we are running under JBoss
     private boolean isJBoss = checkForClass("org.jboss.mx.util.MBeanServerLocator");
+
+    // Optional domain for registering this handler as a mbean
+    private String qualifier;
+
     // private boolean isWebsphere = checkForClass("com.ibm.websphere.management.AdminServiceFactory");
 
+
+    /**
+     * Create a new MBeanServer with no qualifier
+     */
     public MBeanServerHandler() {
+        this(null);
+    }
+
+    /**
+     * Create a new MBeanServer handler who is responsible for managing multiple intra VM {@link MBeanServer} at once
+     *
+     * @param pQualifier optional qualifier used for registering this object as an MBean (can be null)
+     */
+    public MBeanServerHandler(String pQualifier) {
         initMBeanServers();
+        qualifier = pQualifier;
     }
 
     /**
@@ -76,61 +92,54 @@ public class MBeanServerHandler {
                 throw new IllegalStateException("Internal: IOException " + e + ". Shouldn't happen.",e);
             }
         } else {
+            wokaroundJBossBug(pJmxReq);
+            return handleRequest(pRequestHandler, pJmxReq);
+        }
+    }
+
+    private Object handleRequest(JsonRequestHandler pRequestHandler, JmxRequest pJmxReq)
+            throws ReflectionException, MBeanException, AttributeNotFoundException, InstanceNotFoundException {
+        AttributeNotFoundException attrException = null;
+        InstanceNotFoundException objNotFoundException = null;
+        for (MBeanServer s : mBeanServers) {
             try {
-                wokaroundJBossBug(pJmxReq);
-                AttributeNotFoundException attrException = null;
-                InstanceNotFoundException objNotFoundException = null;
-                for (MBeanServer s : mBeanServers) {
-                    try {
-                        return pRequestHandler.handleRequest(s, pJmxReq);
-                    } catch (InstanceNotFoundException exp) {
-                        // Remember exceptions for later use
-                        objNotFoundException = exp;
-                    } catch (AttributeNotFoundException exp) {
-                        attrException = exp;
-                    } catch (IOException exp) {
-                        throw new IllegalStateException("I/O Error while dispatching",exp);
-                    }
-                }
-                if (attrException != null) {
-                    throw attrException;
-                }
-                // Must be there, otherwise we would nave have left the loop
-                throw objNotFoundException;
-            } catch (ReflectionException e) {
-                throw new IllegalStateException("Internal error for '" + pJmxReq.getAttributeName() +
-                        "' on object " + pJmxReq.getObjectName() + ": " + e,e);
-            } catch (MBeanException e) {
-                throw new IllegalStateException("Exception while fetching the attribute '" + pJmxReq.getAttributeName() +
-                        "' on object " + pJmxReq.getObjectName() + ": " + e,e);
+                return pRequestHandler.handleRequest(s, pJmxReq);
+            } catch (InstanceNotFoundException exp) {
+                // Remember exceptions for later use
+                objNotFoundException = exp;
+            } catch (AttributeNotFoundException exp) {
+                attrException = exp;
+            } catch (IOException exp) {
+                throw new IllegalStateException("I/O Error while dispatching",exp);
             }
         }
+        if (attrException != null) {
+            throw attrException;
+        }
+        // Must be there, otherwise we would not have left the loop
+        throw objNotFoundException;
     }
 
     /**
      * Register a MBean under a certain name to the first availabel MBeans server
      *
      * @param pMBean MBean to register
-     * @param pName optional name under which the bean should be registered. If not provided,
+     * @param pOptionalName optional name under which the bean should be registered. If not provided,
      * it depends on whether the MBean to register implements {@link javax.management.MBeanRegistration} or
      * not.
      *
      * @return the name under which the MBean is registered.
      */
-    public ObjectName registerMBean(Object pMBean,String ... pName)
-            throws MalformedObjectNameException, NotCompliantMBeanException, MBeanRegistrationException, InstanceAlreadyExistsException {
+    public ObjectName registerMBean(Object pMBean,String ... pOptionalName)
+            throws MalformedObjectNameException, NotCompliantMBeanException, InstanceAlreadyExistsException {
         if (mBeanServers.size() > 0) {
             Exception lastExp = null;
             for (MBeanServer server : mBeanServers) {
                 try {
-                    if (pName != null && pName.length > 0) {
-                        ObjectName oName = new ObjectName(pName[0]);
-                        return server.registerMBean(pMBean,oName).getObjectName();
-                    } else {
-                        // Needs to implement MBeanRegistration interface
-                        return server.registerMBean(pMBean,null).getObjectName();
-                    }
-                } catch (Exception exp) {
+                    return registerMBeanAtServer(server, pMBean, pOptionalName);
+                } catch (RuntimeException exp) {
+                    lastExp = exp;
+                } catch (MBeanRegistrationException exp) {
                     lastExp = exp;
                 }
             }
@@ -141,6 +150,17 @@ public class MBeanServerHandler {
         throw new IllegalStateException("No MBeanServer initialized yet");
     }
 
+    private ObjectName registerMBeanAtServer(MBeanServer pServer, Object pMBean, String[] pName)
+            throws MalformedObjectNameException, InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
+        if (pName != null && pName.length > 0 && pName[0] != null) {
+            ObjectName oName = new ObjectName(pName[0]);
+            return pServer.registerMBean(pMBean,oName).getObjectName();
+        } else {
+            // Needs to implement MBeanRegistration interface
+            return pServer.registerMBean(pMBean,null).getObjectName();
+        }
+    }
+
     /**
      * Unregisters a MBean under a certain name to the first availabel MBeans server
      *
@@ -296,4 +316,71 @@ public class MBeanServerHandler {
     }
 
 
+    public String mBeanServersInfo() {
+        StringBuffer ret = new StringBuffer();
+        Set<MBeanServer> servers = getMBeanServers();
+
+        ret.append("Found ").append(servers.size()).append(" MBeanServers\n");
+        for (MBeanServer s : servers) {
+            ret.append("    ")
+                    .append("++ ")
+                    .append(s.toString())
+                    .append(": default domain = ")
+                    .append(s.getDefaultDomain())
+                    .append(", ")
+                    .append(s.getMBeanCount())
+                        .append(" MBeans\n");
+
+            ret.append("        Domains:\n");
+            boolean javaLangFound = false;
+            for (String d : s.getDomains()) {
+                if ("java.lang".equals(d)) {
+                    javaLangFound = true;
+                }
+                appendDomainInfo(ret, s, d);
+            }
+            if (!javaLangFound) {
+                // JBoss fails to list java.lang in its domain list
+                appendDomainInfo(ret,s,"java.lang");
+            }
+        }
+        ret.append("\n");
+        ret.append("Platform MBeanServer: ")
+                .append(ManagementFactory.getPlatformMBeanServer())
+                .append("\n");
+        return ret.toString();
+    }
+
+    private void appendDomainInfo(StringBuffer pRet, MBeanServer pServer, String pDomain) {
+        try {
+            pRet.append("         == ").append(pDomain).append("\n");
+            Set<ObjectInstance> beans = pServer.queryMBeans(new ObjectName(pDomain + ":*"),null);
+            for (ObjectInstance o : beans) {
+                String n = o.getObjectName().getCanonicalKeyPropertyListString();
+                pRet.append("              ").append(n).append("\n");
+            }
+        } catch (MalformedObjectNameException e) {
+            // Shouldnt happen
+            pRet.append("              INTERNAL ERROR: ").append(e).append("\n");
+        }
+    }
+
+    // ==============================================================================================
+    // Needed for providing the name for our MBean
+    public ObjectName preRegister(MBeanServer server, ObjectName name) throws MalformedObjectNameException {
+        return new ObjectName(getObjectName());
+    }
+
+    public String getObjectName() {
+        return OBJECT_NAME + (qualifier != null ? "," + qualifier : "");
+    }
+
+    public void postRegister(Boolean registrationDone) {
+    }
+
+    public void preDeregister() {
+    }
+
+    public void postDeregister() {
+    }
 }
