@@ -1,22 +1,18 @@
 package org.jmx4perl.config;
 
-import org.jmx4perl.JmxRequest;
-import org.w3c.dom.Document;
-import org.w3c.dom.Node;
-import org.w3c.dom.NodeList;
-import org.xml.sax.SAXException;
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.*;
+import java.util.regex.Pattern;
 
 import javax.management.MalformedObjectNameException;
 import javax.management.ObjectName;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
-import java.io.IOException;
-import java.io.InputStream;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Map;
-import java.util.Set;
-import java.util.regex.Pattern;
+
+import org.jmx4perl.JmxRequest;
+import org.w3c.dom.*;
+import org.xml.sax.SAXException;
 
 /*
  * jmx4perl - WAR Agent for exporting JMX via JSON
@@ -50,10 +46,7 @@ import java.util.regex.Pattern;
 public class PolicyBasedRestrictor implements Restrictor {
 
     private Set<JmxRequest.Type> typeSet;
-    private Set<ObjectName> patternNameSet;
-    private Map<ObjectName,Set<String>> mBeanReadAttributes;
-    private Map<ObjectName, Set<String>> mBeanWriteAttributes;
-    private Map<ObjectName, Set<String>> mBeanOperations;
+
     private Set<String> allowedHostsSet;
     private Set<String> allowedSubnetsSet;
 
@@ -61,8 +54,15 @@ public class PolicyBasedRestrictor implements Restrictor {
     private static final Pattern IP_PATTERN = Pattern.compile("^[\\d.]+$");
     private static final Pattern SUBNET_PATTERN = Pattern.compile("^[\\d.]+/[\\d.]+$");
 
+    // Configuration for allowed and denied MBean attributes and operations.
+    private MBeanPolicyConfig allow;
+    private MBeanPolicyConfig deny;
+
     public PolicyBasedRestrictor(InputStream pInput) {
         Exception exp = null;
+        if (pInput == null) {
+            throw new SecurityException("No policy file given");
+        }
         try {
             Document doc =
                     DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(pInput);
@@ -76,7 +76,7 @@ public class PolicyBasedRestrictor implements Restrictor {
         catch (MalformedObjectNameException e) { exp = e; }
         finally {
             if (exp != null) {
-                throw new IllegalStateException("Cannot parse policy file",exp);
+                throw new SecurityException("Cannot parse policy file: " + exp,exp);
             }
         }
     }
@@ -89,15 +89,27 @@ public class PolicyBasedRestrictor implements Restrictor {
     }
 
     public boolean isAttributeReadAllowed(ObjectName pName, String pAttribute) {
-        return lookupMBean(mBeanReadAttributes,pName, pAttribute);
+        return check(JmxRequest.Type.READ,pName,pAttribute);
     }
 
     public boolean isAttributeWriteAllowed(ObjectName pName, String pAttribute) {
-        return lookupMBean(mBeanWriteAttributes,pName, pAttribute);
+        return check(JmxRequest.Type.WRITE,pName, pAttribute);
     }
 
     public boolean isOperationAllowed(ObjectName pName, String pOperation) {
-        return lookupMBean(mBeanOperations,pName, pOperation);
+        return check(JmxRequest.Type.EXEC,pName, pOperation);
+    }
+
+    private boolean check(JmxRequest.Type pType, ObjectName pName, String pValue) {
+        if (isTypeAllowed(pType)) {
+            // Its allowed in general, so we only need to check
+            // the denied section, whether its forbidded
+            return deny == null || !matches(deny, pType, pName, pValue);
+        } else {
+            // Its forbidden by default, so we need to check the
+            // allowed section
+            return allow != null && matches(allow, pType, pName, pValue);
+        }
     }
 
     public boolean isRemoteAccessAllowed(String ... pHostOrAddress) {
@@ -121,28 +133,25 @@ public class PolicyBasedRestrictor implements Restrictor {
 
     // ===============================================================================
     // Lookup methods
-
-    private boolean lookupMBean(Map<ObjectName, Set<String>> pMap, ObjectName pName, String pAttribute) {
-        if (pMap == null) {
-            return true;
-        }
-        Set<String> attributes = pMap.get(pName);
-        if (attributes == null) {
-            ObjectName pattern = findMatchingMBeanPattern(pName);
+    private boolean matches(MBeanPolicyConfig pConfig, JmxRequest.Type pType, ObjectName pName, String pValue) {
+        Set<String> values = pConfig.getValues(pType,pName);
+        if (values == null) {
+            ObjectName pattern = pConfig.findMatchingMBeanPattern(pName);
             if (pattern != null) {
-                attributes = pMap.get(pattern);
+                values = pConfig.getValues(pType,pattern);
             }
         }
-        return attributes != null && attributes.contains(pAttribute);
+        return values != null && (values.contains(pValue) || wildcardMatch(values,pValue));
     }
 
-    private ObjectName findMatchingMBeanPattern(ObjectName pName) {
-        for (ObjectName pattern : patternNameSet) {
-            if (pattern.apply(pName)) {
-                return pattern;
+    // Check whether a value matches patterns in pValues
+    private boolean wildcardMatch(Set<String> pValues, String pValue) {
+        for (String pattern : pValues) {
+            if (pattern.contains("*") && pValue.matches(pattern.replaceAll("\\*",".*"))) {
+                return true;
             }
         }
-        return null;
+        return false;
     }
 
 
@@ -171,65 +180,83 @@ public class PolicyBasedRestrictor implements Restrictor {
     }
 
     private void initMBeanSets(Document pDoc) throws MalformedObjectNameException {
-        NodeList nodes = pDoc.getElementsByTagName("mbeans");
-        if (nodes.getLength() > 0) {
-            // Build up maps only if mbeans are given to restrict
-            patternNameSet = new HashSet<ObjectName>();
-            mBeanReadAttributes = new HashMap<ObjectName, Set<String>>();
-            mBeanWriteAttributes = new HashMap<ObjectName, Set<String>>();
-            mBeanOperations = new HashMap<ObjectName, Set<String>>();
+        for (String tag : new String[] { "allow", "mbeans" }) {
+            NodeList nodes = pDoc.getElementsByTagName(tag);
+            if (nodes.getLength() > 0) {
+                // "allow" and "mbeans" are synonyms
+                if (allow == null) {
+                    allow = new MBeanPolicyConfig();
+                }
+                extractMbeanConfiguration(nodes, allow);
+            }
         }
-        for (int i = 0;i<nodes.getLength();i++) {
-            Node node = nodes.item(i);
+        NodeList nodes = pDoc.getElementsByTagName("deny");
+        if (nodes.getLength() > 0) {
+            deny = new MBeanPolicyConfig();
+            extractMbeanConfiguration(nodes,deny);
+        }
+    }
+
+    // Extract configuration and put it into a given MBeanPolicyConfig
+    private void extractMbeanConfiguration(NodeList pNodes,MBeanPolicyConfig pConfig) throws MalformedObjectNameException {
+        for (int i = 0;i< pNodes.getLength();i++) {
+            Node node = pNodes.item(i);
             if (node.getNodeType() != Node.ELEMENT_NODE) {
                 continue;
             }
-            NodeList childs = node.getChildNodes();
-            for (int j = 0;j<childs.getLength();j++) {
-                Node mBeanNode = childs.item(j);
-                if (mBeanNode.getNodeType() != Node.ELEMENT_NODE) {
-                    continue;
-                }
-                assertNodeName(mBeanNode,"mbean");
-                NodeList params = mBeanNode.getChildNodes();
-                String name = null;
-                Set<String> readAttributes = new HashSet<String>();
-                Set<String> writeAttributes = new HashSet<String>();
-                Set<String> operations = new HashSet<String>();
-                for (int k = 0; k<params.getLength(); k++) {
-                    Node param = params.item(k);
-                    if (param.getNodeType() != Node.ELEMENT_NODE) {
-                        continue;
-                    }
-                    assertNodeName(param,"name","attribute","operation");
-                    String tag = param.getNodeName();
-                    if (tag.equals("name")) {
-                        if (name != null) {
-                            throw new IllegalStateException("<name> given twice as MBean name");
-                        } else {
-                            name = param.getTextContent().trim();
-                        }
-                    } else if (tag.equals("attribute")) {
-                        Node mode = param.getAttributes().getNamedItem("mode");
-                        readAttributes.add(param.getTextContent().trim());
-                        if (mode == null || !mode.getNodeValue().equalsIgnoreCase("read")) {
-                            writeAttributes.add(param.getTextContent().trim());
-                        }
-                    } else {
-                        operations.add(param.getTextContent().trim());
-                    }
-                }
-                if (name == null) {
-                    throw new IllegalStateException("No <name> given for <mbean>");
-                }
-                ObjectName oName = new ObjectName(name);
-                if (oName.isPattern()) {
-                    patternNameSet.add(oName);
-                }
-                mBeanReadAttributes.put(oName,readAttributes);
-                mBeanWriteAttributes.put(oName,writeAttributes);
-                mBeanOperations.put(oName,operations);
+            extractPolicyConfig(pConfig, node.getChildNodes());
+        }
+    }
+
+    private void extractPolicyConfig(MBeanPolicyConfig pConfig, NodeList pChilds) throws MalformedObjectNameException {
+        for (int j = 0;j< pChilds.getLength();j++) {
+            Node mBeanNode = pChilds.item(j);
+            if (mBeanNode.getNodeType() != Node.ELEMENT_NODE) {
+                continue;
             }
+            assertNodeName(mBeanNode,"mbean");
+            extractMBeanPolicy(pConfig, mBeanNode);
+        }
+    }
+
+    private void extractMBeanPolicy(MBeanPolicyConfig pConfig, Node pMBeanNode) throws MalformedObjectNameException {
+        NodeList params = pMBeanNode.getChildNodes();
+        String name = null;
+        Set<String> readAttributes = new HashSet<String>();
+        Set<String> writeAttributes = new HashSet<String>();
+        Set<String> operations = new HashSet<String>();
+        for (int k = 0; k < params.getLength(); k++) {
+            Node param = params.item(k);
+            if (param.getNodeType() != Node.ELEMENT_NODE) {
+                continue;
+            }
+            assertNodeName(param,"name","attribute","operation");
+            String tag = param.getNodeName();
+            if (tag.equals("name")) {
+                if (name != null) {
+                    throw new SecurityException("<name> given twice as MBean name");
+                } else {
+                    name = param.getTextContent().trim();
+                }
+            } else if (tag.equals("attribute")) {
+                extractAttribute(readAttributes, writeAttributes, param);
+            } else if (tag.equals("operation")) {
+                operations.add(param.getTextContent().trim());
+            } else {
+                throw new SecurityException("Tag <" + tag + "> invalid");
+            }
+        }
+        if (name == null) {
+            throw new SecurityException("No <name> given for <mbean>");
+        }
+        pConfig.addValues(new ObjectName(name),readAttributes,writeAttributes,operations);
+    }
+
+    private void extractAttribute(Set<String> pReadAttributes, Set<String> pWriteAttributes, Node pParam) {
+        Node mode = pParam.getAttributes().getNamedItem("mode");
+        pReadAttributes.add(pParam.getTextContent().trim());
+        if (mode == null || !mode.getNodeValue().equalsIgnoreCase("read")) {
+            pWriteAttributes.add(pParam.getTextContent().trim());
         }
     }
 
@@ -277,10 +304,53 @@ public class PolicyBasedRestrictor implements Restrictor {
                 buffer.append(",");
             }
         }
-        throw new IllegalStateException(
+        throw new SecurityException(
                 "Expected element " + buffer.toString() + " but got " + pNode.getNodeName());
     }
 
 
+    // ====================================================================================================
 
+    // Class combining various maps for attributes, operations and name patterns
+    private static class MBeanPolicyConfig {
+        private Set<ObjectName> patterns = new HashSet<ObjectName>();
+        private Map<ObjectName,Set<String>> readAttributes = new HashMap<ObjectName, Set<String>>();
+        private Map<ObjectName,Set<String>> writeAttributes = new HashMap<ObjectName, Set<String>>();
+        private Map<ObjectName,Set<String>> operations = new HashMap<ObjectName, Set<String>>();
+
+        public void addPattern(ObjectName pObjectName) {
+            patterns.add(pObjectName);
+        }
+
+        void addValues(ObjectName pOName, Set<String> pReadAttributes, Set<String> pWriteAttributes, Set<String> pOperations) {
+            readAttributes.put(pOName,pReadAttributes);
+            writeAttributes.put(pOName,pWriteAttributes);
+            operations.put(pOName,pOperations);
+            if (pOName.isPattern()) {
+                addPattern(pOName);
+            }
+        }
+
+        Set<String> getValues(JmxRequest.Type pType, ObjectName pName) {
+            if (JmxRequest.Type.READ == pType) {
+                return readAttributes.get(pName);
+            } else if (JmxRequest.Type.WRITE == pType) {
+                return writeAttributes.get(pName);
+            } else if (JmxRequest.Type.EXEC == pType) {
+                return operations.get(pName);
+            } else {
+                throw new IllegalArgumentException("Invalid type " + pType);
+            }
+        }
+
+        ObjectName findMatchingMBeanPattern(ObjectName pName) {
+            // Check all stored patterns for a match and return the pattern if one is found
+            for (ObjectName pattern : patterns) {
+                if (pattern.apply(pName)) {
+                    return pattern;
+                }
+            }
+            return null;
+        }
+    }
 }
