@@ -83,8 +83,12 @@ sub new {
         my $file = $file_or_hash ? $file_or_hash : $ENV{HOME} . "/.j4p";
         if (-e $file) {
             if ($HAS_CONFIG_GENERAL) {
+                local $SIG{__WARN__} = sub {};  # Keep Config::General silent
+                                                # when including things twice
                 $config = {
-                           new Config::General(-ConfigFile => $file,-LowerCaseNames => 1)->getall
+                           new Config::General(-ConfigFile => $file,-LowerCaseNames => 1,
+                                               -UseApacheInclude => 1,-IncludeRelative => 1, -IncludeAgain => 0,
+                                               -IncludeGlob => 1, -IncludeDirectories => 1,  -CComments => 0)->getall
                           };
             } else {
                 warn "Configuration file $file found, but Config::General is not installed.\n" . 
@@ -97,8 +101,10 @@ sub new {
         die "Invalid argument ",$file_or_hash;
     }
     if ($config) {
-        $self->{config} = &_prepare_server_hash($config);
-        $self->{servers} = &_get_configured_servers($config);
+        $self->{server_config} = &_extract_servers($config);
+        $self->{servers} = [ values %{$self->{server_config}} ];
+        map { $self->{$_} = $config->{$_ } } grep { $_ ne "server" } keys %$config;
+        #print Dumper($self);
     }
 
     bless $self,(ref($class) || $class);
@@ -129,7 +135,7 @@ if no such configuration exist.
 sub get_server_config {
     my $self = shift;
     my $name = shift || die "No server name given to reference to get config for";
-    return $self->{config} ? $self->{config}->{$name} : undef;
+    return $self->{server_config} ? $self->{server_config}->{$name} : undef;
 }
 
 =item $servers = $config->get_servers 
@@ -143,24 +149,24 @@ sub get_servers {
     return $self->{servers} || [];
 }
 
-sub _prepare_server_hash {
-    my $config = shift;
-    my $servers = &_get_configured_servers($config);
-    my $ret = {};
-    for my $server (@$servers) {
-        $ret->{$server->{name}} = $server;
-    };
-    return $ret;
-}
-
-sub _get_configured_servers {
+sub _extract_servers {
     my $config = shift;
     my $servers = $config->{server};
-    return [] unless $servers;
-    if (ref($servers) eq "HASH") {
-        return [ $servers ];
-    } else {
+    my $ret = {};
+    return $ret unless $servers;
+    if (ref($servers) eq "ARRAY") {
+        for my $s (@$servers) {
+            die "No name given for server config " . Dumper($s) . "\n" unless $s->{name};
+            $ret->{$s->{name}} = $s;
+        }
+        return $ret;
+    } elsif (ref($servers) eq "HASH") {
+        for my $name (keys %$servers) {
+            $servers->{$name}->{name} = $name;
+        }
         return $servers;
+    } else {
+        die "Invalid configuration type ",ref($servers),"\n";        
     }
 }
 
