@@ -13,8 +13,6 @@ use Carp;
 use Scalar::Util qw(looks_like_number);
 use URI::Escape;
 use Text::ParseWords;
-use JSON;
-
 our $AUTOLOAD;
 
 =head1 NAME
@@ -66,6 +64,7 @@ sub get_requests {
     my $jmx = shift;
     my $args = shift;
 
+    my $request;
     my $do_read = $self->attribute || $self->value;
     if ($self->alias) {
         my $alias = JMX::Jmx4Perl::Alias->by_name($self->alias);
@@ -73,17 +72,11 @@ sub get_requests {
         $do_read = $alias->type eq "attribute";
     }
     my @requests = ();
-    my $request;
     if ($do_read) {
-        $request = JMX::Jmx4Perl::Request->new(READ,$self->_prepare_read_args($jmx));
+        push @requests,JMX::Jmx4Perl::Request->new(READ,$self->_prepare_read_args($jmx));
     } else {
-        $request = JMX::Jmx4Perl::Request->new(EXEC,$self->_prepare_exec_args($jmx,@$args));
+        push @requests,JMX::Jmx4Perl::Request->new(EXEC,$self->_prepare_exec_args($jmx,@$args));
     }
-    my $method = $self->{np}->opts->{method} || $self->{config}->{method};
-    if ($method) {
-        $request->method($method);
-    }
-    push @requests,$request;
 
     if ($self->base) {
         if (!looks_like_number($self->base)) {
@@ -150,7 +143,8 @@ sub extract_responses {
     
     # Normalize value 
     my ($value_conv,$unit) = $self->_normalize_value($value);
-    my $label = $self->_get_name(cleanup => 1);
+    # Common args
+    my $label = "'".$self->_get_name(cleanup => 1)."'";
     if ($self->base) {
         # Calc relative value 
         my $base_value = $self->_base_value($self->base,$responses,$requests);
@@ -164,15 +158,6 @@ sub extract_responses {
                           $self->unit ? (uom => $self->unit) : ());
         # Do the real check.
         my ($code,$mode) = $self->_check_threshhold($rel_value);
-        # For Multichecks, we remember the label of a currently failed check
-        if ($code != OK && $opts->{error_stat}) {
-            my $label = $self->{config}->{key} || $self->{config}->{name};
-            if ($label) {
-                my $arr = $opts->{error_stat}->{$code} || [];
-                push @$arr,$label;
-                $opts->{error_stat}->{$code} = $arr;
-            }
-        }
         my ($base_conv,$base_unit) = $self->_normalize_value($base_value);
         $np->add_message($code,$self->_exit_message(code => $code,mode => $mode,rel_value => $rel_value, 
                                                     value => $value_conv, unit => $unit,base => $base_conv, 
@@ -208,9 +193,7 @@ sub _null_safe_value {
     my $self = shift;
     my $value = shift;
     if (defined($value)) {
-        if (JSON::is_bool($value)) {
-            return "$value";
-        } elsif (ref($value) && $self->string) {
+        if (ref($value) && $self->string) {
             # We can deal with complex values withing string comparison
             if (ref($value) eq "ARRAY") {
                 return join ",",@{$value};
@@ -295,20 +278,18 @@ sub _base_value {
     }
     my $resp = shift @{$responses};
     my $req = shift @{$requests};
-    $np->nagios_die($resp->{error}) if $resp->{error};
     return $self->_extract_value($req,$resp);
 }
 
 # Normalize value if a unit-of-measurement is given.
 
 # Units and how to convert from one level to the next
-my @UNITS = ([ qw(ns us ms s m h d) ],[qw(B KB MB GB TB)]);
+my @UNITS = ([ qw(us ms s m h d) ],[qw(B KB MB GB TB)]);
 my %UNITS = 
   (
-   ns => 1,   
    us => 10**3,
    ms => 10**3,
-   s => 10**3,
+   s => 1,
    m => 60,
    h => 60,
    d => 24,
@@ -330,11 +311,11 @@ sub _normalize_value {
             next unless $units->[$i] eq $unit;
             my $ret = $value;
             my $u = $unit;
-            if (abs($ret) > 1) {
+            if ($ret > 1) {
                 # Go up the scale ...
                 return ($value,$unit) if $i == $#{$units};
                 for my $j ($i+1 .. $#{$units}) {
-                    if (abs($ret / $UNITS{$units->[$j]}) >= 1) {                    
+                    if ($ret / $UNITS{$units->[$j]} >= 1) {                    
                         $ret /= $UNITS{$units->[$j]};
                         $u = $units->[$j];
                     } else {
@@ -345,7 +326,7 @@ sub _normalize_value {
                 # Go down the scale ...
                 return ($value,$unit) if $i == 0;
                 for my $j (reverse(0 .. $i-1)) {
-                    if ($ret < 1) {     
+                    if ($ret <= 1) {     
                         $ret *= $UNITS{$units->[$j+1]};
                         $u = $units->[$j];
                     } else {
@@ -364,13 +345,10 @@ sub _verify_response {
     my ($self,$req,$resp) = @_;
     my $np = $self->{np};
     if ($resp->is_error) {
-        my $stacktrace = $resp->stacktrace;
-        my $extra = "";
-        $extra = ref($stacktrace) eq "ARRAY" ? join "\n",@$stacktrace : $stacktrace if $stacktrace;
-        $np->nagios_die("Error: ".$resp->status." ".$resp->error_text.$extra);
+        $np->nagios_die("Error: ".$resp->status." ".$resp->error_text.($resp->stacktrace ? "\nStacktrace:\n".$resp->stacktrace : ""));
     }
     
-    if (!$req->is_mbean_pattern && (ref($resp->value) && !$self->string) && !JSON::is_bool($resp->value)) { 
+    if (!$req->is_mbean_pattern && (ref($resp->value) && !$self->string)) { 
         $np->nagios_die("Response value is a " . ref($resp->value) .
                         ", not a plain value. Did you forget a --path parameter ?". " Value: " . 
                         Dumper($resp->value));
@@ -385,29 +363,18 @@ sub _get_name {
         if ($self->name) {
             $name = $self->name;
         } else {
-            # Default name, tried to be generated from various parts
-            if ($self->alias) {
-                $name = "[".$self->alias.($self->path ? "," . $self->path : "") ."]";
-            } else {
-                my $val = $self->value;
-                if ($val) {
-                    $name = "[" . $self->value . "]";
-                } else {
-                    my $a_or_o = $self->attribute || $self->operation || "";
-                    my $p = $self->path ? "," . $self->path : "";
-                    $name = "[" . $self->mbean . "," . $a_or_o . $p . "]";
-                }
-            }
+            # Default name
+            $name = $self->alias ? 
+          "[".$self->alias.($self->path ? "," . $self->path : "") ."]" : 
+            $self->value ? 
+              "[" . $self->value . "]" :
+            "[".$self->mbean.",".$self->attribute.($self->path ? "," . $self->path : "")."]";
         }
     }
     if ($args->{cleanup}) {
         # Enable this when '=' gets forbidden
         $name =~ s/=/#/g;
     }
-    # Prepare label for usage with Nagios::Plugin, which will blindly 
-    # add quotes if a space is contained in the label.
-    # We are doing the escape of quotes ourself here
-    $name =~ s/'/''/g;
     return $name;
 }
 
@@ -435,21 +402,8 @@ sub _prepare_exec_args {
     my $np = $self->{np};
     my $jmx = shift;
 
-    # Merge CLI arguments and arguments from the configuration,
-    # with CLI arguments taking precedence
-    my @cli_args = @_;
-    my $config_args = $self->{config}->{args};
-    my @args = ();
-    if ($config_args) {
-        my @c_args = (@$config_args);
-        while (@cli_args || @c_args) {
-            my $cli_arg = shift @cli_args;
-            my $config_arg = shift @c_args;
-            push @args, defined($cli_arg) ? $cli_arg : $config_arg;
-        }
-    } else {
-        @args = @cli_args;
-    }
+    my @args = @_;
+
     if ($self->alias) {
         my @req_args = $jmx->resolve_alias($self->alias);
         $np->nagios_die("Cannot resolve operation alias ",$self->alias()) unless @req_args >= 2;
@@ -462,19 +416,11 @@ sub _prepare_exec_args {
 sub _split_attr_spec {
     my $self = shift;
     my $name = shift;
-    my @ret = ();
-    # Text:ParseWords is used for split on "/" taking into account
-    # quoting and escaping
-    for my $p (&parse_line("/",1,$name)) {
-        # We need to 'unescape' things ourselves
-        # since we want quotes to remain in the names (using '0'
-        # above would kill those quotes, too). 
-        $p =~ s|\\(.)|$1|sg;
-        push @ret,$p;
-    }    
-    return @ret;
+
+    return &parse_line('/',0,$name);
 }
 
+
 sub _check_threshhold {
     my $self = shift;
     my $value = shift;
@@ -609,8 +555,7 @@ sub _format_label {
                 }
             } elsif ($what eq "t") {
                 my $code = $args->{code};
-                my $val = $code == CRITICAL ? $self->critical : ($code == WARNING ? $self->warning : "");
-                $ret .= sprintf $format . "s",defined($val) ? $val : "";
+                $ret .= sprintf $format . "s",$code == CRITICAL ? $self->critical : ($code == WARNING ? $self->warning : "");
             } elsif ($what eq "c") {
                 $ret .= sprintf $format . "s",$STATUS_TEXT{$args->{code}};
             } elsif ($what eq "n") {
@@ -654,7 +599,7 @@ my $CHECK_CONFIG_KEYS = {
                          "label" => "label",
                          # New:
                          "value" => "value",
-                         "null" => "null",
+                         "null" => "null"
                         };
 
 
